/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Declares default methods for the vrpsx_ostreambuf_t interface.
 */

#ifndef VRPSX_OSTREAMBUF_DEFAULT_H
#define VRPSX_OSTREAMBUF_DEFAULT_H 1

#include "../interface/vrpsx_ostreambuf.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* Implements the "put1" operation of an vrpsx_ostreambuf_t by using
 * its buffer and its "flush" operation, which must not call this method,
 * and which must leave room in the buffer for at least one character
 * unless it fails.
 */
int vrpsx_ostreambuf_put1_from_flush(
    vrpsx_ostreambuf_t *osb,
    char ch);

/* Casts &vrpsx_ostreambuf_put1_from_flush for
 * use in a subinterface virtual function table.
 */
#define VRPSX_OSTREAMBUF_PUT1_FROM_FLUSH(TYPE)  \
    ((int (*)(TYPE *, char))                    \
     (&vrpsx_ostreambuf_put1_from_flush))


/* Implements the "putn" operation of an vrpsx_ostreambuf_t by using
 * its buffer and its "put1" operation, which must not call this method.
 */
int vrpsx_ostreambuf_putn_from_put1(
    vrpsx_ostreambuf_t *osb,
    char const *begin,
    size_t size);

/* Casts &vrpsx_ostreambuf_putn_from_put1 for
 * use in a subinterface virtual function table.
 */
#define VRPSX_OSTREAMBUF_PUTN_FROM_PUT1(TYPE)   \
    ((int (*)(TYPE *, char const *, size_t))    \
     (&vrpsx_ostreambuf_putn_from_put1))


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_OSTREAMBUF_DEFAULT_H */
